from holoviews import Store, extension
from holoviews.core.options import Compositor
from holoviews.operation.element import contours

from ..element import Contours, Polygons


if hasattr(extension, 'register_backend_callback'):
    def _load_bokeh():
        from . import bokeh # noqa
    extension.register_backend_callback('bokeh', _load_bokeh)

    def _load_mpl():
        from . import mpl # noqa
    extension.register_backend_callback('matplotlib', _load_mpl)

    backends = Store.loaded_backends()
    if 'bokeh' in backends:
        _load_bokeh()
    if 'matplotlib' in backends:
        _load_mpl()
else:
    try:
        from . import mpl # noqa
    except ImportError:
        pass

    try:
        from . import bokeh # noqa
    except ImportError:
        pass

Compositor.register(Compositor("LineContours", contours, None,
                               'data', transfer_options=True,
                               transfer_parameters=True,
                               output_type=Contours,
                               backends=['bokeh', 'matplotlib']))
Compositor.register(Compositor("FilledContours", contours.instance(filled=True),
                               None, 'data', transfer_options=True,
                               transfer_parameters=True,
                               output_type=Polygons,
                               backends=['bokeh', 'matplotlib']))



from unittest import TestCase

from holoviews.element.comparison import Comparison as HvComparison

from .geo import Image, Points, LineContours, FilledContours

class Comparison(HvComparison):

    @classmethod
    def register(cls):
        super(Comparison, cls).register()
        cls.equality_type_funcs[Image] = cls.compare_dataset
        cls.equality_type_funcs[Points] = cls.compare_dataset
        cls.equality_type_funcs[LineContours] = cls.compare_dataset
        cls.equality_type_funcs[FilledContours] = cls.compare_dataset
        return cls.equality_type_funcs


class ComparisonTestCase(Comparison, TestCase):
    """
    Class to integrate the Comparison class with unittest.TestCase.
    """

    def __init__(self, *args, **kwargs):
        TestCase.__init__(self, *args, **kwargs)
        registry = Comparison.register()
        for k, v in registry.items():
            self.addTypeEqualityFunc(k, v)
